/***************************************************************************
                          crrc_aero.c  -  description
                             -------------------
        X       Aerodynamic force, lbs, in X-axis (+ forward)
        Y       Aerodynamic force, lbs, in Y-axis (+ right)
        Z       Aerodynamic force, lbs, in Z-axis (+ down)
        L       Aero. moment about X-axis (+ roll right), ft-lbs
        M       Aero. moment about Y-axis (+ pitch up), ft-lbs
        N       Aero. moment about Z-axis (+ nose right), ft-lbs

        0       Subscript implying initial, or nominal, value
        u       X-axis component of airspeed (ft/sec) (+ forward)
        v       Y-axis component of airspeed (ft/sec) (+ right)
        w       Z-axis component of airspeed (ft/sec) (+ down)
        p       X-axis ang. rate (rad/sec) (+ roll right), rad/sec
        q       Y-axis ang. rate (rad/sec) (+ pitch up), rad/sec
        r       Z-axis ang. rate (rad/sec) (+ yaw right), rad/sec
        beta    Angle of sideslip, degrees (+ wind in RIGHT ear)
        da      Aileron deflection, degrees (+ left ail. TE down)
        de      Elevator deflection, degrees (+ trailing edge down)
        dr      Rudder deflection, degrees (+ trailing edge LEFT)
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "ls_types.h"
#include "ls_generic.h"
#include "ls_cockpit.h"

/* semispan location for computing local CL */

extern COCKPIT cockpit_;
extern char airplane_filename[256];

extern DATA  C_ref;  //  reference chord (ft)
extern DATA  B_ref;  //  reference span  (ft)
extern DATA  S_ref;  //  reference area (ft^2)
extern DATA  U_ref;  //  reference speed for Re-scaling of CD_prof  (ft/s)

extern DATA  Alpha_0;  //  baseline alpha (rad)
extern DATA  Cm_0   ;  //  baseline Cm at Alpha_0
extern DATA  CL_0   ;  //  baseline CL at Alpha_0
extern DATA  CL_max ;  //  positive stall limit
extern DATA  CL_min ;  //  negative stall limit

extern DATA  CD_prof;  //  profile CD at U_ref
extern DATA  Uexp_CD;  //  for Re-scaling of CD_prof  ~ (U/U_ref)^Uexp_CD

extern DATA  CL_a;  //  lift-force   / alpha    ~  2 pi / (1 + 2/AR)
extern DATA  Cm_a;  //  pitch-moment / alpha    (pitch stability)

extern DATA  CY_b;  //  side-force  / sideslip
extern DATA  Cl_b;  //  roll-moment / sideslip (crucial for rudder-only turns)
extern DATA  Cn_b;  //  yaw-moment  / sideslip (yaw stability)

extern DATA  CL_q;  //  lift-force   / pitch-rate
extern DATA  Cm_q;  //  pitch-moment / pitch-rate  (pitch damping)

extern DATA  CY_p;  //  side-force  / roll-rate
extern DATA  Cl_p;  //  roll-moment / roll-rate   (roll damping)
extern DATA  Cn_p;  //  yaw-moment  / roll-rate   (yaw-roll coupling)

extern DATA  CY_r;  //  side-force  / yaw-rate
extern DATA  Cl_r;  //  roll-moment / yaw-rate
extern DATA  Cn_r;  //  yaw-moment  / yaw-rate  (yaw damping)

extern DATA  CL_de;  //  lift-force   / elevator
extern DATA  Cm_de;  //  pitch-moment / elevator

extern DATA  CY_dr; // side-force  / rudder
extern DATA  Cl_dr; // roll-moment / rudder
extern DATA  Cn_dr; // yaw-moment  / rudder

extern DATA  CY_da;  // side-force  / aileron 
extern DATA  Cl_da;  // roll-moment / aileron
extern DATA  Cn_da;  // yaw-moment  / aileron

extern DATA eta_loc;
extern DATA CG_arm;
extern DATA CL_drop;
extern DATA CD_stall;
extern int stalling;

/* new CD model       MD 8 June 01
   non-unity span efficiency
   profile CD is quadratic on CL and aileron deflection
   CDi is increased by load deformation due to aileron and roll rate  */
     
extern DATA span_eff; //  span efficiency
extern DATA CL_CD0;   //  CL at minimum profile CD
extern DATA CD_CLsq;  //  d(CD)/d(CL^2),  curvature of parabolic profile polar
extern DATA CD_AIsq;  //  d(CD)/d(aileron^2) , curvature of ail. CD influence
extern DATA CD_ELsq;  //  d(CD)/d(elevator^2), curvature of ele. CD influence

void aero(SCALAR dt, int Initialize)
// Calculate forces and moments for the current time step.  If Initialize is
// zero, then re-initialize coefficients by reading in the coefficient file.
{
 static int init = 0;
 static SCALAR elevator, aileron, rudder;

 SCALAR Phat, Qhat, Rhat;  // dimensionless rotation rates
 SCALAR CL_left, CL_cent, CL_right; // CL values across the span
 SCALAR dCL_left,dCL_cent,dCL_right; // stall-induced CL changes
 SCALAR dCD_left,dCD_cent,dCD_right; // stall-induced CD changes
 SCALAR dCl,dCn,dCl_stall,dCn_stall;  // stall-induced roll,yaw moments
 SCALAR dCm_stall;  // Stall induced pitching moment.
 SCALAR CL_wing, CD_all, CD_scaled, Cl_w;
 SCALAR Cl_r_mod,Cn_p_mod;
 SCALAR CL,CD,Cx,Cy,Cz,Cl,Cm,Cn;
 SCALAR QS;
 SCALAR G_11,G_12,G_13;
 SCALAR G_21,G_22,G_23;
 SCALAR G_31,G_32,G_33;
 SCALAR U_body_X,U_body_Y,U_body_Z;
 SCALAR V_body_X,V_body_Y,V_body_Z;
 SCALAR W_body_X,W_body_Y,W_body_Z;
 SCALAR P_atmo,Q_atmo,R_atmo;

 if (!init)
  {
   init = -1;
  }  

  elevator = Long_control + Long_trim;
  aileron  = Lat_control;
  rudder   = Rudder_pedal;

//  printf("%f\n",V_rel_wind);

  /* compute gradients of Local velocities w.r.t. Body coordinates
      G_11  =  dU_local/dx_body
      G_12  =  dU_local/dy_body   etc.  */

  G_11 = U_atmo_X*T_local_to_body_11
       + U_atmo_Y*T_local_to_body_12
       + U_atmo_Z*T_local_to_body_13;
  G_12 = U_atmo_X*T_local_to_body_21
       + U_atmo_Y*T_local_to_body_22
       + U_atmo_Z*T_local_to_body_23;
  G_13 = U_atmo_X*T_local_to_body_31
       + U_atmo_Y*T_local_to_body_32
       + U_atmo_Z*T_local_to_body_33;

  G_21 = V_atmo_X*T_local_to_body_11
       + V_atmo_Y*T_local_to_body_12
       + V_atmo_Z*T_local_to_body_13;
  G_22 = V_atmo_X*T_local_to_body_21
       + V_atmo_Y*T_local_to_body_22
       + V_atmo_Z*T_local_to_body_23;
  G_23 = V_atmo_X*T_local_to_body_31
       + V_atmo_Y*T_local_to_body_32
       + V_atmo_Z*T_local_to_body_33;

  G_31 = W_atmo_X*T_local_to_body_11
       + W_atmo_Y*T_local_to_body_12
       + W_atmo_Z*T_local_to_body_13;
  G_32 = W_atmo_X*T_local_to_body_21
       + W_atmo_Y*T_local_to_body_22
       + W_atmo_Z*T_local_to_body_23;
  G_33 = W_atmo_X*T_local_to_body_31
       + W_atmo_Y*T_local_to_body_32
       + W_atmo_Z*T_local_to_body_33;

  //printf("%f %f %f %f\n",W_atmo_X,W_atmo_Y,G_31,G_32);

  /* now compute gradients of Body velocities w.r.t. Body coordinates */
  /*  U_body_x  =  dU_body/dx_body   etc.  */

  U_body_X = T_local_to_body_11*G_11
           + T_local_to_body_12*G_21
           + T_local_to_body_13*G_31;
  U_body_Y = T_local_to_body_11*G_12
           + T_local_to_body_12*G_22
           + T_local_to_body_13*G_32;
  U_body_Z = T_local_to_body_11*G_13
           + T_local_to_body_12*G_23
           + T_local_to_body_13*G_33;
			      
  V_body_X = T_local_to_body_21*G_11
           + T_local_to_body_22*G_21
           + T_local_to_body_23*G_31;
  V_body_Y = T_local_to_body_21*G_12
           + T_local_to_body_22*G_22
           + T_local_to_body_23*G_32;
  V_body_Z = T_local_to_body_21*G_13
           + T_local_to_body_22*G_23
           + T_local_to_body_23*G_33;
			      
  W_body_X = T_local_to_body_31*G_11
           + T_local_to_body_32*G_21
           + T_local_to_body_33*G_31;
  W_body_Y = T_local_to_body_31*G_12
           + T_local_to_body_32*G_22
           + T_local_to_body_33*G_32;
  W_body_Z = T_local_to_body_31*G_13
           + T_local_to_body_32*G_23
           + T_local_to_body_33*G_33;

   /* set rotation rates of airmass motion */
  P_atmo =  W_body_X;
  Q_atmo = -W_body_Y;
  R_atmo =  V_body_Z;

  //  printf("%f %f %f\n",P_atmo,Q_atmo,R_atmo);
  if (V_rel_wind != 0)
   {
  /* set net effective dimensionless rotation rates */
    Phat = (P_body - P_atmo) * B_ref / (2.0*V_rel_wind);
    Qhat = (Q_body - Q_atmo) * C_ref / (2.0*V_rel_wind);
    Rhat = (R_body - R_atmo) * B_ref / (2.0*V_rel_wind);
   }
  else
   {
    Phat=0;
    Qhat=0;
    Rhat=0;
   }

//  printf("Phat: %f Qhat: %f Rhat: %f\n",Phat,Qhat,Rhat);

    /* compute local CL at three spanwise locations */
  CL_left  = CL_0 + CL_a*(Alpha - Alpha_0 - Phat*eta_loc);
  CL_cent  = CL_0 + CL_a*(Alpha - Alpha_0               );
  CL_right = CL_0 + CL_a*(Alpha - Alpha_0 + Phat*eta_loc);

// printf("CL_left: %f CL_cent: %f CL_right: %f\n",CL_left,CL_cent,CL_right);

    /* set CL-limit changes */
  dCL_left  = 0.;
  dCL_cent  = 0.;
  dCL_right = 0.;

  stalling=0;
  if (CL_left  > CL_max)
    {
      dCL_left  = CL_max-CL_left -CL_drop; 
      stalling=1;
    }

  if (CL_cent  > CL_max)
    {
      dCL_cent  = CL_max-CL_cent -CL_drop;
      stalling=1;
    }

  if (CL_right > CL_max)
    {
      dCL_right = CL_max-CL_right -CL_drop;
      stalling=1;      
    }

  if (CL_left  < CL_min)
    {
      dCL_left  = CL_min-CL_left -CL_drop;
      stalling=1;
    }

  if (CL_cent  < CL_min)
    {
      dCL_cent  = CL_min-CL_cent -CL_drop; 
      stalling=1;
    }

  if (CL_right < CL_min)
    {
      dCL_right = CL_min-CL_right -CL_drop;
      stalling=1;
    }

  /* set average wing CL */
  CL_wing = CL_0 + CL_a*(Alpha-Alpha_0)
          + 0.25*dCL_left + 0.5*dCL_cent + 0.25*dCL_right;

//  printf("CL_wing: %f\n",CL_wing);


  /* correct profile CD for CL dependence and aileron dependence */
  CD_all = CD_prof
         + CD_CLsq*(CL_wing-CL_CD0)*(CL_wing-CL_CD0)
         + CD_AIsq*aileron*aileron
         + CD_ELsq*elevator*elevator;

//  printf("CD_all:%lf\n",CD_all);

    /* scale profile CD with Reynolds number via simple power law */
 if (V_rel_wind > 0.1)
 {
  CD_scaled = CD_all*pow(((double)V_rel_wind/(double)U_ref),Uexp_CD);
 }
 else
 {
	CD_scaled=CD_all;
 }

//  printf("CD_scaled:%lf\n",CD_scaled);



  /* Scale lateral cross-coupling derivatives with wing CL */
  Cl_r_mod = Cl_r*CL_wing/CL_0;
  Cn_p_mod = Cn_p*CL_wing/CL_0;

  //  printf("Cl_r_mod: %f Cn_p_mod: %f\n",Cl_r_mod,Cn_p_mod);

    /* total average CD with induced and stall contributions */
  dCD_left  = CD_stall*dCL_left *dCL_left ;
  dCD_cent  = CD_stall*dCL_cent *dCL_cent ;
  dCD_right = CD_stall*dCL_right*dCL_right;

  /* total CL, with pitch rate and elevator contributions */
  CL = (CL_wing + CL_q*Qhat + CL_de*elevator)*Cos_alpha;

//  printf("CL:%f\n",CL);

    /* assymetric stall will cause roll and yaw moments */
  dCl =  0.45*-1*(dCL_right-dCL_left)*0.5*eta_loc;
  dCn =  0.45*(dCD_right-dCD_left)*0.5*eta_loc;
  dCm_stall = (0.25*dCL_left + 0.5*dCL_cent + 0.25*dCL_right)*CG_arm;

//  printf("dCl: %f dCn:%f\n",dCl,dCn);

    /* stall-caused moments in body axes */
  dCl_stall = dCl*Cos_alpha - dCn*Sin_alpha;
  dCn_stall = dCl*Sin_alpha + dCn*Cos_alpha;

    /* total CD, with induced and stall contributions */

  Cl_w = Cl_b*Beta  + Cl_p*Phat + Cl_r_mod*Rhat
       + dCl_stall  + Cl_da*aileron;
  CD = CD_scaled
     + (CL*CL + 32.0*Cl_w*Cl_w)*S_ref/(B_ref*B_ref*3.14159*span_eff)
     + 0.25*dCD_left + 0.5*dCD_cent + 0.25*dCD_right;

  //printf("CL: %f CD:%f L/D:%f\n",CL,CD,CL/CD);

    /* total forces in body axes */
  Cx = -CD*Cos_alpha + CL*Sin_alpha*Cos_beta*Cos_beta;
  Cz = -CD*Sin_alpha - CL*Cos_alpha*Cos_beta*Cos_beta;
  Cy = CY_b*Beta  + CY_p*Phat + CY_r*Rhat + CY_dr*rudder;

    /* total moments in body axes */
  Cl =        Cl_b*Beta  + Cl_p*Phat + Cl_r_mod*Rhat + Cl_dr*rudder
           + dCl_stall                               + Cl_da*aileron;
  Cn =        Cn_b*Beta  + Cn_p_mod*Phat + Cn_r*Rhat + Cn_dr*rudder
           + dCn_stall                               + Cn_da*aileron;
  Cm = Cm_0 + Cm_a*(Alpha-Alpha_0) +dCm_stall
                         + Cm_q*Qhat                 + Cm_de*elevator;

    /* set dimensional forces and moments */
  QS = 0.5*Density*V_rel_wind*V_rel_wind * S_ref;
  
  F_X_aero = Cx * QS;
  F_Y_aero = Cy * QS;
  F_Z_aero = Cz * QS;

  M_l_aero = Cl * QS * B_ref;
  M_m_aero = Cm * QS * C_ref;
  M_n_aero = Cn * QS * B_ref;
//  printf("%f %f %f %f %f %f\n",F_X_aero,F_Y_aero,F_Z_aero,M_l_aero,M_m_aero,M_n_aero);
}
